package com.tiangua.star.util;

import com.alibaba.fastjson.JSONObject;

import java.security.MessageDigest;
import java.util.Map;
import java.util.TreeMap;

public class SignUtil {

    /**
     * 生成签名，startTime 为当前 Unix 时间戳（秒），endTime 为 8 小时后的时间戳。
     *
     * @param sid   请求参数 sid
     * @param brand 请求参数 brand
     * @param key   加密密钥
     * @return 计算得到的 MD5 签名字符串
     */
    public static JSONObject generateSign(int sid, int brand, String key) {
        // 获取当前 Unix 时间戳（秒）
        long currentTimestamp = System.currentTimeMillis() / 1000;
        // 8小时后的 Unix 时间戳
        long endTimestamp = currentTimestamp + 8 * 3600;

        // 使用 TreeMap 自动按 key 的自然顺序排序
        Map<String, String> requestData = new TreeMap<>();
        requestData.put("sid", String.valueOf(sid));
        requestData.put("brand", String.valueOf(brand));
        requestData.put("startTime", String.valueOf(currentTimestamp));
        requestData.put("endTime", String.valueOf(endTimestamp));
        System.out.println(requestData);
        // 拼接 key=value 的字符串，多个参数之间用 &
        StringBuilder sb = new StringBuilder();
        for (Map.Entry<String, String> entry : requestData.entrySet()) {
            sb.append(entry.getKey())
              .append("=")
              .append(entry.getValue())
              .append("&");
        }
        // 去掉末尾的 & 符号
        if (sb.length() > 0) {
            sb.deleteCharAt(sb.length() - 1);
        }

        // 拼接密钥
        String dataToHash = sb.toString() + key;
        String sign = md5(dataToHash);

        JSONObject json = new JSONObject();
        json.put("sid", sid);
        json.put("brand", brand);
        json.put("startTime", currentTimestamp);
        json.put("endTime", endTimestamp);
        json.put("sign", sign);

        return json;
    }

    /**
     * 计算字符串的 MD5 值（小写）
     *
     * @param input 待加密字符串
     * @return MD5 加密后的字符串
     */
    public static String md5(String input) {
        try {
            MessageDigest md = MessageDigest.getInstance("MD5");
            byte[] messageDigest = md.digest(input.getBytes("UTF-8"));
            // 将字节数组转换为十六进制字符串
            StringBuilder hexString = new StringBuilder();
            for (byte b : messageDigest) {
                String hex = Integer.toHexString(0xff & b);
                if (hex.length() == 1) {
                    hexString.append('0');
                }
                hexString.append(hex);
            }
            return hexString.toString();
        } catch (Exception e) {
            throw new RuntimeException("MD5 加密失败", e);
        }
    }

    /**
     * 生成签名，包含 sid、brand 和 mobile 参数
     *
     * @param sid    请求参数 sid
     * @param brand  请求参数 brand
     * @param mobile 请求参数 mobile 数组
     * @param key    密钥
     * @return 计算得到的 MD5 签名字符串
     */
    public static String generateSignWithMobile(int sid, int brand, String[] mobile, String key) {
        // 使用 TreeMap 自动按 key 的自然顺序排序
        Map<String, String> requestData = new TreeMap<>();
        requestData.put("sid", String.valueOf(sid));
        requestData.put("brand", String.valueOf(brand));
        // 如果 mobile 为数组，则使用逗号连接各个手机号
        String mobileStr = String.join(",", mobile);
        requestData.put("mobile", mobileStr);

        // 拼接所有参数为 "key=value" 格式，参数之间使用 &
        StringBuilder sb = new StringBuilder();
        for (Map.Entry<String, String> entry : requestData.entrySet()) {
            sb.append(entry.getKey())
                    .append("=")
                    .append(entry.getValue())
                    .append("&");
        }
        // 去掉末尾的 & 符号
        if (sb.length() > 0) {
            sb.deleteCharAt(sb.length() - 1);
        }

        // 拼接密钥
        String dataToHash = sb.toString() + key;
        // 计算 MD5 签名并返回
        return md5(dataToHash);
    }

    /**
     * 计算字符串的 MD5 值（小写）
     *
     * @param input 待加密字符串
     * @return MD5 加密后的字符串
     */
    public static String md52(String input) {
        try {
            MessageDigest md = MessageDigest.getInstance("MD5");
            byte[] messageDigest = md.digest(input.getBytes("UTF-8"));
            // 将字节数组转换为十六进制字符串
            StringBuilder hexString = new StringBuilder();
            for (byte b : messageDigest) {
                String hex = Integer.toHexString(0xff & b);
                if (hex.length() == 1) {
                    hexString.append('0');
                }
                hexString.append(hex);
            }
            return hexString.toString();
        } catch (Exception e) {
            throw new RuntimeException("MD5 加密失败", e);
        }
    }

    // 测试方法
    public static void main(String[] args) {
        int sid = 81;
        int brand = 43;
        String key = "730969D3F673BA42432C574378B9A8F1";

        JSONObject sign = generateSign(sid, brand, key);
        System.out.println("生成的签名: " + sign);

//        int sid = 81;
//        int brand = 43;
//        String[] mobile = {"13303412254", "18763878111"};
//        String key = "730969D3F673BA42432C574378B9A8F1";
//
//        String sign = generateSignWithMobile(sid, brand, mobile, key);
//        System.out.println("生成的签名: " + sign);
    }
}
